#include <libxml/tree.h>
#include <libxml/xpath.h>
#include <gconf/gconf-client.h>
#include <stdlib.h>
#include <string.h>

// the functionnality functions of the program
int dump_from_gconf ();
int load_from_file (char*);

// the helper functions
xmlNodePtr ab_from_xml(char *);
xmlNodePtr ab_from_gconf ();
int gconf_from_ab (xmlNodePtr);
int xml_from_ab (xmlNodePtr);
void print_ab (xmlNodePtr);

// things of interest
#define CONTACTS_KEY        "/apps/gnomemeeting/contacts/"
#define CONTACTS_GROUPS_KEY "/apps/gnomemeeting/contacts/groups/"

// the main function
int
main(int argc, char *argv[])
{
  switch (argc)
    {
    case 1:
      return dump_from_gconf ();
      break;
    case 2:
      return load_from_file (argv[1]);
      break;
    }

  fprintf (stderr, "Usage:\n");
  fprintf (stderr, "%s            dumps addressbook from gconf\n", argv[0]);
  fprintf (stderr, "%s <filename> creates gconf addressbook from xml file\n", argv[0]);
  return EXIT_FAILURE;
}

// functionnality functions

int
dump_from_gconf ()
{
  xmlNodePtr ab = ab_from_gconf ();
  if (ab)
    return xml_from_ab (ab);
  else
    return EXIT_FAILURE;
}

int
load_from_file (char *filename)
{
  fprintf (stderr, "Trying to load from %s\n", filename);
  xmlNodePtr ab = ab_from_xml (filename);
  if (ab)
    {
      return gconf_from_ab (ab);
    } else
      return EXIT_FAILURE;
}

// helper functions

xmlNodePtr
ab_from_xml (char *filename)
{
  xmlDocPtr ab = NULL;
  ab = xmlParseFile (filename);
  if (!ab)
    {
      fprintf (stderr, "Couldn't load the file!\n");
      return NULL;
    }
	
  return ab->children;
}

xmlNodePtr
ab_from_gconf ()
{
  xmlNodePtr ab = xmlNewNode (NULL, "addressbook");
  xmlNodePtr group = NULL;
  xmlNodePtr contact = NULL;

  GConfClient *client = gconf_client_get_default ();

  GSList *groups = NULL;
  GSList *groups_iter = NULL;
  gchar *group_name;

  gchar *contacts_key = NULL;
  GSList *contacts = NULL;
  GSList *contacts_iter = NULL;

  char **contact_info = NULL;
  gchar *name = NULL;
  gchar *url = NULL;
  gchar *speed_dial = NULL;

  groups = gconf_client_get_list (client,
				  CONTACTS_KEY "groups_list",
				  GCONF_VALUE_STRING, NULL);
  groups_iter = groups;
  while (groups_iter)
    {
      if (groups_iter->data)
	{
	  group_name = gconf_unescape_key ((char *)groups_iter->data, -1);
	  group = xmlNewNode (NULL, "group");
	  xmlSetProp (group, "name", group_name);
	  contacts_key =  g_strdup_printf ("%s%s", CONTACTS_GROUPS_KEY,
					   g_utf8_strdown ((char *)groups_iter->data, -1));
	  contacts = gconf_client_get_list (client, contacts_key, GCONF_VALUE_STRING, NULL);
	  contacts_iter = contacts;
	  while (contacts_iter)
	    {
	      contact_info = g_strsplit ((char *)contacts_iter->data, "|", 0);
	      name = contact_info[0];
	      url = contact_info[1];
	      speed_dial = contact_info[2];

	      contact = xmlNewNode (NULL, "contact");
	      xmlNewTextChild (contact, NULL, "name", name);
	      xmlNewTextChild (contact, NULL, "url", url);
	      if (speed_dial)
		xmlNewTextChild (contact, NULL, "speed_dial", speed_dial);

	      xmlAddChild (group, contact);
 
	      contacts_iter = contacts_iter->next;
	    }
	}

      xmlAddChild (ab, group);
      groups_iter = groups_iter->next;
    }

  return ab;
}

void
print_ab (xmlNodePtr ab)
{
  for (xmlNodePtr group = ab->children; group; group = group->next)
    {
      fprintf (stderr, "Group name: %s\n", xmlGetProp(group, "name"));
      for (xmlNodePtr contact = group->children; contact; contact = contact->next)
	{
	  if (!strcmp(contact->name, "contact"))
	    {
	      fprintf (stderr, "  contact\n");
	      for (xmlNodePtr info = contact->children; info; info = info->next)
		{
		  if (info->type == XML_ELEMENT_NODE)
		    {
		      fprintf (stderr, "    %s: %s\n", info->name,
			       (info->children->content != NULL)?(char*)info->children->content:"(null)");
		    }
		}
	    }
	}
    }
}

int
gconf_from_ab (xmlNodePtr ab)
{
  GConfClient *client = gconf_client_get_default ();
  
  GSList *groups = NULL;
  for (xmlNodePtr group = ab->children; group; group = group->next)
    {
      gchar *group_name = xmlGetProp (group, "name");
      groups = g_slist_append (groups, group_name);
      gchar *contacts_key =  g_strdup_printf ("%s%s", CONTACTS_GROUPS_KEY,
					      g_utf8_strdown (group_name, -1));
      GSList *contacts = NULL;
      for (xmlNodePtr contact = group->children; contact; contact = contact->next)
	{
	  if (!strcmp(contact->name, "contact"))
	    {
	      gchar *name = NULL;
	      gchar *url = NULL;
	      gchar *speed_dial = NULL;
	      for (xmlNodePtr info = contact->children; info; info = info->next)
		{
		  if (info->type == XML_ELEMENT_NODE)
		    {
		      if (!strcmp (info->name, "name"))
			name = info->children->content;
		      else if (!strcmp (info->name, "url"))
			url = info->children->content;
		      else if (!strcmp (info->name, "speed_dial"))
			speed_dial = info->children->content;
		    }
		}
	      if (name && url)
		{
		  char *contact_info = NULL;
		  if (speed_dial)
		    contact_info = g_strdup_printf ("%s|%s|%s", name, url, speed_dial);
		  else
		    contact_info = g_strdup_printf ("%s|%s", name, url);
		  contacts = g_slist_append (contacts, contact_info);
		}
	    }
	} // loop on contacts
      gconf_client_set_list (client, contacts_key, GCONF_VALUE_STRING, contacts, NULL);
    } // loop on groups
  gconf_client_set_list (client, CONTACTS_KEY "groups_list", GCONF_VALUE_STRING, groups, NULL);
  return 0;
}

int
xml_from_ab (xmlNodePtr ab)
{
  xmlDocPtr doc = xmlNewDoc ("1.0");
  xmlAddChild (doc, ab);
  xmlDocDump (stdout, doc);
  return 0;
}
